#!/usr/bin/perl

package Cognition::Export::Address;
use Cognition::Export;

# Reference:
# http://www.ibiblio.org/hhalpin/homepage/notes/vcardtable.html
# (Though that page does contain a number of mistakes, such as equating a vCard
# "TITLE" [a job title] with a FOAF "title" [e.g. Mr, Mrs].)

sub is
{
	my $subject = shift;
	return
		unless (defined $subject->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type'));
	foreach my $Type ( @{$subject
		->property('http://www.w3.org/1999/02/22-rdf-syntax-ns#type')
		->valObjects} )
	{
		my $type = $Type->toString;
		return 1
			if (($type eq 'http://www.w3.org/2001/vcard-rdf/3.0#adr')
			||  ($type eq 'http://www.w3.org/2006/vcard/ns#Address')
			||  ($type eq 'http://www.w3.org/2006/03/hcard#adr')
			||  ($type eq 'http://www.w3.org/2000/10/swap/pim/contact#Address')
			||  ($type eq 'urn:ietf:rfc:2426#adr'));
	}
	return;
}

sub outputs
{
	return [
		{
			code => 'VCFLine',
			name => 'vCard-compatible Line',
			type => 'text/x-directory-part'
		},
		{
			code => 'Text',
			name => 'Plain text address (one line)',
			type => 'text/plain'
		}
	];
}

sub outText
{
	my ($key, $adr) = outVCFLine(@_);
	
	$adr =~ s/\;/\, /g;
	$adr =~ s/\\\,/\,/g;
	while ($adr =~ /\, \,/)
		{ $adr =~ s/\, \,/\,/g; }
	$adr =~ s/(^(\, )+|(\, )+$)//g;
	
	return $adr;
}

sub outVCFLine
{
	my $triples = shift;
	my $subject = shift;
	my $S = $triples->{$subject};

	my @addr;
	$addr[0] = [allOf($S, 'urn:ietf:rfc:2426#post-office-box',
		'http://www.w3.org/2006/vcard/ns#post-office-box',
		'http://www.w3.org/2006/03/hcard#post-office-box',
		'http://www.w3.org/2001/vcard-rdf/3.0#post-office-box')];
	$addr[1] = [allOf($S, 'urn:ietf:rfc:2426#extended-address',
		'http://www.w3.org/2006/vcard/ns#extended-address',
		'http://www.w3.org/2006/03/hcard#extended-address',
		'http://www.w3.org/2001/vcard-rdf/3.0#extended-address',
		'http://www.w3.org/2000/10/swap/pim/contact#addressProperty')];
	$addr[2] = [allOf($S, 'urn:ietf:rfc:2426#street-address',
		'http://www.w3.org/2006/vcard/ns#street-address',
		'http://www.w3.org/2006/03/hcard#street-address',
		'http://www.w3.org/2001/vcard-rdf/3.0#street-address',
		'http://www.w3.org/2000/10/swap/pim/contact#street')];
	$addr[3] = [allOf($S, 'urn:ietf:rfc:2426#locality',
		'http://www.w3.org/2006/vcard/ns#locality',
		'http://www.w3.org/2006/03/hcard#locality',
		'http://www.w3.org/2001/vcard-rdf/3.0#locality',
		'http://www.w3.org/2000/10/swap/pim/contact#city')];
	$addr[4] = [allOf($S, 'urn:ietf:rfc:2426#region',
		'http://www.w3.org/2006/vcard/ns#region',
		'http://www.w3.org/2006/03/hcard#region',
		'http://www.w3.org/2001/vcard-rdf/3.0#region')];
	$addr[5] = [allOf($S, 'urn:ietf:rfc:2426#postal-code',
		'http://www.w3.org/2006/vcard/ns#postal-code',
		'http://www.w3.org/2006/03/hcard#postal-code',
		'http://www.w3.org/2001/vcard-rdf/3.0#postal-code')];
	$addr[6] = [allOf($S, 'urn:ietf:rfc:2426#country-name',
		'http://www.w3.org/2006/vcard/ns#country-name',
		'http://www.w3.org/2006/03/hcard#country-name',
		'http://www.w3.org/2001/vcard-rdf/3.0#country-name',
		'http://www.w3.org/2000/10/swap/pim/contact#country')];
		
	my @address;
	foreach my $A (@addr)
	{
		my @line;
		foreach my $a (@$A)
		{
			$a =~ s/\,/\\\,/g;
			$a =~ s/\;/\\\;/g;
			push @line, $a;
		}
		
		push @address, (join ', ', @line);
	}
	my $address = join ';', @address;
		
	@types = allOf($S, 'urn:ietf:rfc:2426#type',
		'http://www.w3.org/2006/vcard/ns#type',
		'http://www.w3.org/2006/03/hcard#type',
		'http://www.w3.org/2001/vcard-rdf/3.0#type');
	my $types;
	foreach my $t (@types)
	{
		$t =~ s/(^\s+|\s+$)//g;
		$t =~ s/\s+/-/g;
		$types .= ",".uc($t);
	}
	$types =~ s/^\,//;
	
	$key = 'ADR';
	$key = "ADR;TYPE=$types"
		if (length $types);
		
	return ($key, $address);
}

1;
