#!/usr/bin/perl

package Cognition::Export::CalComponent;
use Cognition::Export;
use Cognition::Export::Alarm;
use Cognition::Export::BlogEntry;
use Cognition::Export::Event;
use Cognition::Export::FBInfo;
use Cognition::Export::Todo;
use HTML::Entities qw(encode_entities_numeric);;

sub is
{
	my $s = shift;
	return Cognition::Export::Alarm::is($s)
		||Cognition::Export::FBInfo::is($s)
		||Cognition::Export::BlogEntry::is($s)
		||Cognition::Export::Todo::is($s)
		||Cognition::Export::Event::is($s);
}

sub outputs
{
	return [
		{
			code  => 'ICS',
			name  => 'iCalendar',
			type  => 'text/directory',
			multi => 1
		}
	];
}

sub outICS
{
	return "BEGIN:VCALENDAR\r\n"
		. _outICS(@_)
		. "END:VCALENDAR\r\n";
}

sub multiICS
{
	my $triples = shift;
	my $rv = '';
	foreach my $s (keys %$triples)
	{
		$rv .= _outICS($triples, $s)
			if (defined $triples->{$s} && is($triples->{$s}));
	}

	return "BEGIN:VCALENDAR\r\n"
		. $rv
		. "END:VCALENDAR\r\n";
}

sub _outICS
{
	my $triples = shift;
	my $sURL    = shift;
	my $sObj    = $triples->{$sURL};

	if (Cognition::Export::Event::is($sObj))
		{ return Cognition::Export::Event::_outICS($triples, $sURL); }

	if (Cognition::Export::BlogEntry::is($sObj))
		{ return Cognition::Export::BlogEntry::_outICS($triples, $sURL); }

	if (Cognition::Export::Todo::is($sObj))
		{ return Cognition::Export::Todo::_outICS($triples, $sURL); }

	if (Cognition::Export::Alarm::is($sObj))
		{ return Cognition::Export::Alarm::_outICS($triples, $sURL); }
		
	if (Cognition::Export::FBInfo::is($sObj))
		{ return Cognition::Export::FBInfo::_outICS($triples, $sURL); }
}

1;
