#!/usr/bin/perl

######################################################################
package Cognition::HTMLParser::Utils;
######################################################################

use XML::LibXML qw(:all);
use strict;
use Text::Wrap;
use CGI::Util;

BEGIN
{
	our $VERSION   = 0.1;	
	our @outline   = ('outline:', 0, 0, 0, 0, 0, 0);
	our $lastLevel = 0;
}


sub searchClass
{
	my $target = shift;
	my $dom    = shift;
	
	my @matches = ();
	foreach my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('class');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /x)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchClass


sub searchRel
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/[\:\.]/\[\:\.\]/;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $classList = $node->getAttribute('rel');
		next unless (length $classList);
		
		if ($classList =~ / (^|\s) $target (\s|$) /ix)
		{
			push @matches, $node;
		}
	}
	
	return @matches;
	
} #/sub searchRel


sub searchID
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/^\#//;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $id   = $node->getAttribute('id') || next;
		
		if ($id eq $target)
		{
			return $node;
		}
	}
	
} #/sub searchID


sub searchCXP
{
	my $target = shift;
	my $dom    = shift;
	
	$target =~ s/^\#//;
	
	my @matches = ();
	for my $node ($dom->getElementsByTagName('*'))
	{
		my $id   = $node->getAttribute('_xpath') || next;
		
		if ($id eq $target)
		{
			return $node;
		}
	}
	
} #/sub searchCXP


sub outline_point
{
	my $level = shift;
	$level =~ s/[^0-9]//g;
	
	if ($Cognition::HTMLParser::Utils::lastLevel < $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel < $level)
		{
			$Cognition::HTMLParser::Utils::lastLevel++;
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
		}
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel > $level)
	{
		while ($Cognition::HTMLParser::Utils::lastLevel > $level)
		{
			$Cognition::HTMLParser::Utils::outline[$Cognition::HTMLParser::Utils::lastLevel] = 1;
			$Cognition::HTMLParser::Utils::lastLevel--;
		}
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
	
	elsif ($Cognition::HTMLParser::Utils::lastLevel == $level)
	{
		$Cognition::HTMLParser::Utils::outline[$level]++;
	}
		
	my @rv = ();
	for (my $i=1; $i<=$level; $i++)
	{
		push @rv, $Cognition::HTMLParser::Utils::outline[$i];
	}
	
	return join('.', @rv);
}


sub STRINGIFY
{
	my $str = _STRINGIFY(@_);
	
	# \x1D is used as a "soft" line break. It can be "absorbed" into an adjacent
	# "hard" line break.
	$str =~ s/\x1D+/\x1D/g;
	$str =~ s/\x1D\n/\n/gs;
	$str =~ s/\n\x1D/\n/gs;
	$str =~ s/\x1D/\n/gs;
	
	$str =~ s/(^\s+|\s+$)//gs;
	return $str;
}


sub _STRINGIFY
{
	my $domNode   = shift || return;
	my $inPRE     = shift || 0;
	my $indent    = shift || 0;
	my $rv = '';
	
	my $tag;
	if ($domNode->nodeType == XML_ELEMENT_NODE)
	{
		$tag = lc($domNode->tagName);
	}
	
	# Change behaviour within <pre>.
	$inPRE++ if ($tag eq 'pre');
	
	# Text node, or equivalent.
	if (!$tag || $tag eq 'img' || $tag eq 'input')
	{
		$rv = $domNode->getData               if (!$tag);
		$rv = $domNode->getAttribute('alt')   if ($tag
		                                      && length $domNode->getAttribute('alt'));
		$rv = $domNode->getAttribute('value') if ($tag
		                                      && length $domNode->getAttribute('value'));

		unless ($inPRE)
		{
			$rv =~ s/[\s\r\n]+/ /gs;
			$rv = ''
				unless ($rv =~ /[^\s]/s);	
		}
		
		return $rv;
	}
	
	# Breaks.
	return "\n" if ($tag eq 'br');
	return "\x1D\n====\n\n"
		if ($tag eq 'hr');
		
	# Deleted text.
	return '' if ($tag eq 'del');

	# Get stringified children.
	my (@parts, @ctags);
	my $extra = 0;
	if ($tag =~ /^([oud]l|blockquote)$/)
	{
		$extra += 6; # Advisory for word wrapping.
	}
	foreach my $child ($domNode->getChildNodes)
	{
		my $ctag = $child->nodeType==XML_ELEMENT_NODE ? lc($child->tagName) : undef;
		my $str  = _STRINGIFY($child, $inPRE, $indent + $extra);
		push @ctags, $ctag;
		push @parts, $str;
	}
	
	if ($tag eq 'ul')
	{
		$rv .= "\x1D";
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'li');
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n/\n    /gs;
			$rv .= "  * $_\n";
		}
		$rv .= "\n";
	}
	
	elsif ($tag eq 'ol')
	{
		$rv .= "\x1D";
		my $count = 0;
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'li');
			$count++;
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n/\n    /gs;
			$rv .= sprintf("\% 2d. \%s\n", $count, $_);
		}
		$rv .= "\n";
	}

	elsif ($tag eq 'dl')
	{
		$rv .= "\x1D";
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i] eq 'dt' || $ctags[$i] eq 'dd');
			
			if ($ctags[$i] eq 'dt')
			{
				$rv .= $parts[$i] . ':';
				$rv =~ s/\:\:$/\:/;
				$rv .= "\n";
			}
			elsif ($ctags[$i] eq 'dd')
			{
				$_ = $parts[$i];
				s/(^\x1D|\x1D$)//g;
				s/\x1D+/\x1D/g;
				s/\x1D\n/\n/gs;
				s/\n\x1D/\n/gs;
				s/\x1D/\n/gs;
				s/\n/\n    /gs;
				$rv .= sprintf("    \%s\n\n", $_);
			}
		}
	}

	elsif ($tag eq 'blockquote')
	{
		$rv .= "\x1D";
		for (my $i=0; defined $parts[$i]; $i++)
		{
			next unless ($ctags[$i]);
			
			$_ = $parts[$i];
			s/(^\x1D|\x1D$)//g;
			s/\x1D+/\x1D/g;
			s/\x1D\n/\n/gs;
			s/\n\x1D/\n/gs;
			s/\x1D/\n/gs;
			s/\n\n/\n/;
			s/\n/\n>   /gs;
			$rv .= ">   $_\n";
		}
		$rv =~ s/>   $/\x1D/;
	}
	else
	{
		$rv = join '', @parts;
		
		if ($tag =~ /^(p|h[1-9]?|div|address|li|dd|dt|tr|caption|table)$/ && !$inPRE)
		{
			$rv =~ s/^[\t ]//s;
			local($Text::Wrap::columns);
			$Text::Wrap::columns = 78 - $indent;
			$rv = "\x1D".Text::Wrap::wrap('','',$rv);
			if ($tag =~ /^(p|h[1-9]?)$/)
			{
				$rv .= "\n\n";
			}
		}
		
		if ($tag eq 'sub')
			{ $rv = "($rv)"; }
		elsif ($tag eq 'sup')
			{ $rv = "[$rv]"; }
		elsif ($tag eq 'q')
			{ $rv = "\"$rv\""; }
		elsif ($tag eq 'th' || $tag eq 'td')
			{ $rv = "$rv\t"; }
	}

	return $rv;
}


sub xmlns
{
	my $ns  = shift;
	my $dom = shift;
	
	while ($dom->nodeType==XML_ELEMENT_NODE)
	{
		if (length $dom->getAttribute("xmlns:$ns"))
		{
			return $dom->getAttribute("xmlns:$ns");
		}
		$dom = $dom->getParentNode;
	}
	
	if ($dom->nodeType==XML_DOCUMENT_NODE)
	{
		if (length $dom->getDocumentElement->getAttribute("xmlns:$ns"))
		{
			return $dom->getDocumentElement->getAttribute("xmlns:$ns");
		}
	}
	
	return undef;
}

sub PrepareXPointers
# We save the node paths into special @_xpath attributes. Why? Because we often
# need to move DOM nodes about (e.g. uF include pattern) but still want access
# to the original node paths. Sometimes we're just dealing with document
# fragments, which have no meaningful node paths. This seems like an inefficient
# way of doing it, but I can't find a better solution.
{
	my $dom = shift;
	foreach my $tag ($dom->getElementsByTagName('*'))
		{ $tag->setAttribute('_xpath', $tag->nodePath); }
}

sub XPointer
{
	my $dom  = shift;
	my $page = shift || '';
	
	my $xpath = $dom->getAttribute('_xpath');
	if (!length $xpath)
		{ $xpath = $dom->nodePath; }
	
	my $escapedXPath = CGI::Util::escape($xpath);
	$escapedXPath =~ s|\%2f|/|ig;
	$escapedXPath =~ s|\%3f|?|ig;
	$escapedXPath =~ s|\%5b|[|ig;
	$escapedXPath =~ s|\%5d|]|ig;
	return "$page#xpointer($escapedXPath)";
}


1; #/package Cognition::HTMLParser::Utils




